@@ -20,10 +20,10 @@ from typing import Optional
 
 import requests
 
-SUPPORTED_TYPE_REGISTRY_PRESETS = ('canvas', 'default', 'development', 'kusama', 'polkadot', 'rococo',
+SUPPORTED_TYPE_REGISTRY_PRESETS = ('canvas', 'default', 'development', 'kusama', 'polkadot', 'rococo', 'metadata_types',
                                    'substrate-node-template', 'westend', 'statemint', 'statemine', 'test')
 
-ONLINE_BASE_URL = 'https://raw.githubusercontent.com/polkascan/py-scale-codec/v0.11/scalecodec/type_registry/'
+ONLINE_BASE_URL = 'https://raw.githubusercontent.com/polkascan/py-scale-codec/v1.0/scalecodec/type_registry/'
 
 
 def load_type_registry_preset(name: str, use_remote_preset: bool = False) -> Optional[dict]:
